/*
 * file:        ADC.h
 * purpose:     class for analog digital conversion
 * project:     c++ framework for avr controllers
 * author:      Reinhard Mantey
 * copyright:   2003-2016 (c) Creative commons - by-nc-sa - some rights reserved
 *              see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#include "ADC.h"

using namespace srd;

volatile struct ADConverter::Values ADConverter::values;


ADConverter::ADConverter(RefVoltage reference, Prescaler prescale, int8_t leftAdjusted) {
  reset();
  if (leftAdjusted) ADMUX |=   1 << ADLAR;
  else              ADMUX &= ~(1 << ADLAR);
  ADMUX  &= 0x1F;
  ADMUX  |= reference << 6;
  ADCSRA &= 0xF8;
  ADCSRA |= prescale | 1 << ADIE;
  }


void ADConverter::enableChannel(Pin pin, int8_t enable) {
  pin.toInput();
  if (enable) values.activeMuxes |=   1 << pin.getPin();
  else        values.activeMuxes &= ~(1 << pin.getPin());
  };


// ADC interrupt service routine
void ADConverter::conversionComplete() {
  int i=values.activeChannel + 1;

  values.v[2] = 13;                                        //TODO: remove test aid
  ++values.completed;                                      //TODO: remove test aid

  /*
   * save ADC result
   */
  if (NUM_OVERSAMPLING > 1)
     values.temp[values.cOversampling][values.activeChannel] = ADC;
  else
     values.v[values.activeChannel] = ADC;

  /*
   * determine next input channel
   */
  for (values.activeChannel = -1; i < 8; ++i) {
      if (values.activeMuxes & (1 << i)) {
         values.activeChannel = i;
         break;
         }
      }

  if (values.activeChannel < 0) {                          // wrap channels
     if (NUM_OVERSAMPLING > 1) {                           // oversampling active?
        if (++values.cOversampling >= NUM_OVERSAMPLING) {  // reached max oversampling
           calcValues();
           values.cOversampling = 0;
           ++values.completed;                             // trigger work done
           }
        }
     else ++values.completed;                              // without oversampling work is always done

     for (i=0; i < 8; ++i) {                               // we stil need a new channel
         if (values.activeMuxes & (1 << i)) {
            values.activeChannel = i;
            break;
            }
         }
     }

  setActiveChannel(values.activeChannel);                  // set mux of adc to new channel
  }


void ADConverter::calcValues() {
  int32_t  scratch;

  for (int n=0; n < 8; ++n) {
      scratch = 0;
      for (int i=0; i < NUM_OVERSAMPLING; ++i) {
          scratch += values.temp[i][n];
          }
      values.v[n] = scratch / NUM_OVERSAMPLING;
      }
  ++values.completed;
  }


void ADConverter::reset() {
  values.activeChannel = -1;
  selectNextChannel();
  for (int n=0; n < 8; ++n) {
      values.v[n] = 0;
      for (int i=0; i < NUM_OVERSAMPLING; ++i) {
          values.temp[i][n] = 0;
          }
      }
  }


void ADConverter::selectNextChannel() {
  int i = values.activeChannel + 1;

  for (values.activeChannel = -1; i < 8; ++i) {
      if (values.activeMuxes & (1 << i)) {
         values.activeChannel = i;
         break;
         }
      }

  if (values.activeChannel < 0) {                          // wrap channels
     if (NUM_OVERSAMPLING > 1) {                           // oversampling active?
        if (++values.cOversampling >= NUM_OVERSAMPLING) {  // reached max oversampling
           calcValues();
           values.cOversampling = 0;
           ++values.completed;                             // trigger work done
           }
        }
     else ++values.completed;                              // without oversampling work is always done

     for (i=0; i < 8; ++i) {                               // we stil need a new channel
         if (values.activeMuxes & (1 << i)) {
            values.activeChannel = i;
            break;
            }
         }
     }

  setActiveChannel(values.activeChannel);                  // set mux of adc to new channel
  }
