/*
 * file:        PushButton.C
 * purpose:     class for timebased interrupt (16bit timer).
 *              clients can be registered to be executed from interrupt
 *              default usage is to use the system interrupt with a millisecond as time base
 * project:     c++ framework for avr controllers
 * author:      Reinhard Mantey
 * copyright:   2003-2016 (c) Creative commons - by-nc-sa - some rights reserved
 *              see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#include "SystemTicker.h"

using namespace srd;

SystemTicker*     SystemTicker::ticker;
         uint16_t SystemTicker::maxTicks;
volatile uint16_t SystemTicker::ticks;
volatile uint32_t SystemTicker::seconds;


SystemTicker::SystemTicker(uint16_t interruptsPerSecond) {
  if (!ticker) {
     ticker = this;
     // only the first call shall setup the 16bit timer!
     setupTimer(interruptsPerSecond);
     }
  };


// setup the 16bit Timer as fast pwm timer with ICR1 as top
// overflow-interrupt will be the system tick handler and execute the static function run()
// timer frequency is set by interruptsPerSecond constructor parameter,
// but the timer can be used as 16bit fast pwm timer any way.
// PWM-output will be enabled and disabled by enablePwmOutput[AB]
// Its not allowed to stop the timer or change prescaler settings.
void SystemTicker::setupTimer(uint16_t interruptsPerSecond) {
  maxTicks = interruptsPerSecond;
#ifdef __AVR_ATmega16__
  ICR1 = F_CPU / interruptsPerSecond;

  // mode 14: fast pwm with icr1 as top
  TCCR1A |= 0 << WGM10 | 1 << WGM11;
  TCCR1B |= 1 << WGM12 | 1 << WGM13
          | 1 << CS10;                                     // no prescaler, timer starts here
  TIMSK  |= 1 << TOIE1;                                    // with ICR1 as top we have overflow at top
#endif
  sei();
  };


void SystemTicker::run() {
  if (!ticker) return;
  if (++ticks > maxTicks) {
     ticks = 0;
     ++seconds;
     }
  };


void SystemTicker::enablePwmOutputA(int8_t enable, int8_t inverted) {
#ifdef __AVR_ATmega16__
  if (enable) {
     if (inverted) {
        TCCR1A |=   1 << COM1A0;
        TCCR1A |=   1 << COM1A1;
        }
     else {
        TCCR1A &= ~(1 << COM1A0);
        TCCR1A |=   1 << COM1A1;
        }
     }
  else {
     TCCR1A &= ~(1 << COM1A1 | 1 << COM1A0);
     }
#endif
  };


void SystemTicker::enablePwmOutputB(int8_t enable, int8_t inverted) {
#ifdef __AVR_ATmega16__
  if (enable) {
     if (inverted) {
        TCCR1A |=   1 << COM1B0;
        TCCR1A |=   1 << COM1B1;
        }
     else {
        TCCR1A &= ~(1 << COM1B0);
        TCCR1A |=   1 << COM1B1;
        }
     }
  else {
     TCCR1A &= ~(1 << COM1B1 | 1 << COM1B0);
     }
#endif
  };

